unit ImageWindow;
{ a unit that can display a grid of images taken from a TImageList
  Draws a focusframe around the selected image, returns index when clicked
  Copyright  1997 by Peter Thrnqvist; all rights reserved
  Contact: NMA96PTH@STUDENT2.LU.SE

}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics,
  Controls, Forms, Dialogs,ExtCtrls,CommCtrl;

type
  TMargin=2..24;
  TImageWindow = class(TGraphicControl)
  private
    { Private declarations }
    FImageList   :TImageList;
    FIndex,
    OldX,
    OldY,
    imWidth,
    imHeight     :integer;
    FBkColor,
    FForeColor,
    FGridColor   :TColor;
    FMargin      :TMargin;
    FNumCols,
    FNumPics     :Cardinal;
    FUseFrame,
    FGrid,FGhost
                 :Boolean;
    procedure DrawFocusFrame(X,Y:integer);
    procedure DrawGrid;
    procedure SetBackColor(Value:TColor);
    procedure SetForeColor(Value:TColor);
    procedure SetGridColor(Value:TColor);
    procedure SetMargin(Value:TMargin);
    procedure SetNumCols(Value:cardinal);
    procedure SetNumPics(Value:cardinal);
    procedure SetUseFrame(Value:Boolean);
    procedure SetGrid(Value:boolean);
    procedure SetGhost(Value:boolean);
    procedure SetImageList(Value:TImageList);
  protected
    { Protected declarations }
    procedure Paint;override;
    procedure MouseMove(Shift:TShiftState;X,Y:integer);override;
    procedure MouseUp(Button:TMouseButton;Shift:TShiftState;X,Y:integer);override;
  public
    { Public declarations }
    constructor Create(AOwner:TComponent);override;
    destructor Destroy;override;
    property ImageIndex:integer read FIndex default 0;  { read-only }
  published
    { Published declarations }
    property BackColor:TColor read FBkColor write SetBackColor default clWindow;
    property ForeColor:TColor read FForeColor write SetForeColor default clWindowText;
    property GhostFrame:boolean read FGhost write SetGhost;
    property Margin: TMargin read FMargin write SetMargin default 2;
    property NumCols:Cardinal read FNumCols write SetNumCols default 4;
    property LastPic:Cardinal read FNumPics write SetNumPics default 0;
    property ImageList:TImageList read FImageList write SetImageList;
    property UseFrame:boolean read FUseFrame write SetUseFrame default True;
    property Grid:boolean read FGrid write SetGrid default True;
    property GridColor:TColor read FGridColor write SetGridColor default clActiveCaption;
    property Width default 64;
    property Height default 64;
    property Align;
    property Visible;
    property Enabled;
    property DragCursor;
    property DragMode;
    property PopUpMenu;
    property ParentShowHint;
    property Hint;
    property ShowHint;
    property OnClick;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnStartDrag;
  end;


  { TImageBox }
  TImageBox = class(TGraphicControl)
  private
    FHiColor,
    FBkColor,
    TmpColor:TColor;
    FBorderStyle:TBorderStyle;
    FImageList:TImageList;
    FIndex:integer;
    FOnEnter:TNotifyEvent;
    FOnExit:TNotifyEvent;
    FDown:boolean;
    FShowClick:boolean;
    procedure SetBkColor(Value:TColor);
    procedure SetHiColor(Value:TColor);
    procedure SetBorderStyle(Value:TBorderStyle);
    procedure SetIndex(Value:integer);
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);override;
    procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
  protected
    procedure PaintFrame;virtual;
    procedure Paint;override;
  public
    constructor Create(AOwner:TComponent);override;
    destructor  Destroy;override;
  published
    property HiColor:TColor read FHiColor write SetHiColor default clActiveCaption;
    property BackColor:TColor read FBkColor write SetBkColor default clWindow;
    property BorderStyle:TBorderStyle read FBorderStyle write SetBorderStyle default bsSingle;
    property ImageList:TImageList read FImageList write FImageList;
    property ImageIndex:integer read FIndex write SetIndex default 0;
    property ShowClick:boolean read FShowClick write FShowClick default False;
    property Width default 36;
    property Height default 36;
    property Align;
    property Text;
    property Visible;
    property Enabled;
    property DragCursor;
    property DragMode;
    property PopUpMenu;
    property ParentShowHint;
    property Hint;
    property ShowHint;
    property OnEnter : TNotifyEvent read FOnEnter write FOnEnter;
    property OnExit : TNotifyEvent read FOnExit write FOnExit;
    property OnClick;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnStartDrag;
  end;

procedure Register;

implementation

{ TImageWindow }
constructor TImageWindow.Create(AOwner:TComponent);
begin
  inherited Create(AOwner);
  FBkColor := clWindow;
  FForeColor := clWindowText;
  FNumCols := 4;
  FUseFrame := True;
  FGhost := False;
  FGrid := True;
  FIndex := 0;
  FMargin := 2;
  OldX := -1;
  OldY := -1;
  SetBounds(10,10,64,64);
end;

destructor TImageWindow.Destroy;
begin
  inherited Destroy;
end;

procedure TImageWindow.Paint;
var i,X,Y:integer;
begin
  with Canvas do
  begin
    Brush.Color := FBkColor;
    Pen.Color := FForeColor;
    if csDesigning in ComponentState then
      Rectangle(0,0,Width,Height)
    else
      FillRect(ClientRect);
  end;

  if Assigned(FImageList) then
  begin
    if FGrid then DrawGrid;
    if FNumCols > FNumPics + 1 then FNumCols := FNumPics + 1;

    X := 0;
    Y := 0;
    if FNumPics > FImageList.Count - 1 then
      SetNumPics(FImageList.Count - 1);
    for i := 0 to FNumPics do
    begin
      FImageList.Draw(Canvas,X + FMargin * 2,Y + FMargin * 2,i);
      Inc(X,imWidth + FMargin * 2);
      if (i mod FNumCols = FNumCols - 1) then
      begin
        Inc(Y,imHeight + FMargin * 2);
        X := 0;
      end;
    end;
  end;
end;

procedure TImageWindow.DrawFocusFrame(X,Y:integer);
var iWidth,iHeight:integer;Rec:TRect;Leaving:boolean;FRows:integer;
begin
  Leaving := False;

  iWidth := imWidth + FMargin * 2;
  iHeight := imHeight + FMargin * 2;
  { get index for X and Y }

  X := Trunc( X / iWidth);
  Y := Trunc( Y / iHeight);

  FRows := (FNumPics div FNumCols);
  { inside bounds ? }
  { special case FRows = 0 }
  if (Y > FRows) and (FRows < 1) then
    Leaving := True
  else
  if (X > FNumCols - 1) or (X + Y * FNumCols > FNumPics)
       or ((Y > FRows) and (FNumPics mod FRows = 0)) then
    Leaving := True;

  { get new starting points }
  X := X * (iWidth);
  Y := Y * (iHeight);

  { erase old frame }
  if ((OldX <> X) or (OldY <> Y)) and (OldX <> - 1) then
  begin
    if FGrid then
      Canvas.Brush.Color := FGridColor
    else
      Canvas.Brush.Color := FBkColor;
    Rec := Rect(FMargin,FMargin,iWidth + FMargin + 1,iHeight + FMargin + 1);
    OffsetRect(Rec,OldX,OldY);
    Canvas.FrameRect(Rec);
{    InflateRect(Rec,-1,-1);
    Canvas.FrameRect(Rec); }
  end;
  if Leaving then Exit;

  Canvas.Brush.Color := FForeColor;
  Rec := Rect(FMargin,FMargin,iWidth + FMargin + 1,iHeight + FMargin + 1);
  OffsetRect(Rec,X,Y);
  Canvas.FrameRect(Rec);
  if FGhost and FGrid then
  begin
    InflateRect(Rec,-1,-1);
    Canvas.FrameRect(Rec);
  end;
  OldX := X;
  OldY := Y;
end;

procedure TImageWindow.DrawGrid;
var i,j,hBuf,wBuf:integer;
begin
  if not Assigned(FImageList) then Exit;

  i := imWidth;
  j := imHeight;

  wBuf := imWidth;
  hBuf := imHeight;

  while i < Width + wBuf do
  begin
    while j < Height + hBuf do
    begin
      DrawFocusFrame(i,j);
      Inc(j,imWidth);
    end;
    Inc(i,imHeight);
    j := imHeight;
  end;
end;

procedure TImageWindow.MouseMove(Shift:TShiftState;X,Y:integer);
begin
  inherited MouseMove(Shift,X,Y);
  if FUseFrame and Assigned(FImageList) then
    DrawFocusFrame(X,Y);
end;

procedure TImageWindow.MouseUp(Button:TMouseButton;Shift:TShiftState;X,Y:integer);
var iWidth,iHeight:integer;
begin
  if Assigned(FImageList) then
  begin
    iWidth := imWidth + FMargin * 2;
    iHeight := imHeight + FMargin * 2;

    { get index for X and Y }
    X := Trunc( X / iWidth);
    Y := Trunc( Y / iHeight);
    { convert to imageindex }
    FIndex := X + Y * FNumCols;
    if FIndex > FNumPics then FIndex := FNumPics;
    if FIndex < 0 then FIndex := 0;
  end;

  if Assigned(OnClick) then OnClick(self);
//  inherited MouseUp(Button,Shift,X,Y);
end;

{ draw a ghost frame too }
procedure TImageWindow.SetGhost(Value:boolean);
begin
  if FGhost <> Value then
  begin
    FGhost := Value;
    Invalidate;
  end;
end;

procedure TImageWindow.SetImageList(Value:TImageList);
begin
  if FImageList <> Value then
  begin
    FImageList := Value;
    if Assigned(FImageList) then
    begin
      imWidth := FImageList.Width;
      imHeight := FImageLIst.Height;
    end
    else
    begin
      imWidth := 16;
      imHeight := 16;
    end;
    Invalidate;
  end;
end;

{
procedure TImageWindow.WMEraseBkgnd(var M : TWMEraseBkgnd);
begin
  M.Result := LRESULT(False);
end;
}
procedure TImageWindow.SetBackColor(Value:TColor);
begin
  if FBkColor <> Value then
  begin
    FBkColor := Value;
    Invalidate;
  end;
end;

procedure TImageWindow.SetForeColor(Value:TColor);
begin
  if FForeColor <> Value then
  begin
    FForeColor := Value;
    Invalidate;
  end;
end;

procedure TImageWindow.SetGridColor(Value:TColor);
begin
  if FGridColor <> Value then
  begin
    FGridColor := Value;
    Invalidate;
  end;
end;

procedure TImageWindow.SetMargin(Value:TMargin);
begin
  if FMargin <> Value then
  begin
    FMargin := Value;
    Invalidate;
  end;
end;

{
  Also resets NumRows to match:
  if Pics > Rows * Cols then Rows will be forced to Rows  = Pics / Cols
}
procedure TImageWindow.SetNumCols(Value:cardinal);
begin
  if FNumCols <> Value then
  begin
    if FNumCols > Value then
    begin
      Canvas.Brush.Color := clBtnFace;
      Canvas.FillRect(Rect(0,0,Width,Height));
    end;  
    FNumCols := Value;
    if FNumCols < 1 then FNumCols := 1;

    Invalidate;
  end;
end;

procedure TImageWindow.SetNumPics(Value:cardinal);
begin
  if FNumPics <> Value then
  begin
    FNumPics := Value;
    Invalidate;
  end;
end;

procedure TImageWindow.SetUseFrame(Value:Boolean);
begin
  if FUseFrame <> Value then
  begin
    FUseFrame := Value;
    Invalidate;
  end;
end;

procedure TImageWindow.SetGrid(Value:boolean);
begin
  if FGrid <> Value then
  begin
    FGrid := Value;
    Invalidate;
  end;
end;


{ TImageBox }


constructor TImageBox.Create(AOwner:TComponent);
begin
  inherited Create(AOwner);
  FHiColor := clActiveCaption;
  FBkColor := clWindow;
  TmpColor := FBkColor;
  FIndex := 0;
  FDown := False;
  FShowClick := False;
  Width := 36;
  Height := 36;
  FBorderStyle := bsSingle;
end;

destructor  TImageBox.Destroy;
begin
  inherited Destroy;
end;

procedure TImageBox.PaintFrame;
var R:TRect;
begin
  R := GetClientRect;
  if FDown and FShowClick then
  begin
    Frame3d(Canvas,R,cl3DDkShadow,cl3DDkShadow,1);
    Frame3d(Canvas,R,clBtnHighLight,clBtnHighLight,1);
    Frame3d(Canvas,R,cl3DDkShadow,cl3DDkShadow,1);
  end
  else
  begin
    Frame3d(Canvas,R,clBtnFace,clBtnFace,1);
    Frame3d(Canvas,R,clBtnShadow,clBtnHighLight,1);
    Frame3d(Canvas,R,cl3DDkShadow,clBtnFace,1);
  end;
end;

procedure TImageBox.Paint;
var R:TRect;dX,dY:integer;
begin
  R := Rect(0,0,Width,Height);

  if FBorderStyle = bsSingle then
  begin
    PaintFrame;
    InflateRect(R,-3,-3);
  end;

  { fill in the rest }
  with Canvas do
  begin
    Brush.Color := TmpColor;
    Brush.Style := bsSolid;
    FillRect(R);
  end;

  if Assigned(FImageList) then
  begin
    { draw in middle }
    dX := (Width - FImageList.Width) div 2;
    dY := (Height - FImageList.Height) div 2;
    FImageList.Draw(Canvas,dX,dY,FIndex);
//    ImageList_Draw(FImageList.Handle,FIndex,dX,dY,Canvas.Handle,ILD_TRANSPARENT);
  end;
end;


procedure TImageBox.SetBkColor(Value:TColor);
begin
  if FBkColor <> Value then
  begin
    FBkColor := Value;
    TmpColor := FBkColor;
    Repaint;
  end;
end;

procedure TImageBox.SetHiColor(Value:TColor);
begin
  if FHiColor <> Value then
  begin
    FHiColor := Value;
    Repaint;
  end;
end;

procedure TImageBox.SetBorderStyle(Value:TBorderStyle);
begin
  if FBorderStyle <> Value then
  begin
    FBorderStyle := Value;
    Repaint;
  end;
end;

procedure TImageBox.SetIndex(Value:integer);
begin
  if FIndex <> Value then
  begin
    FIndex := Value;
    Repaint;
  end;
end;

procedure TImageBox.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  inherited MouseUp(Button,Shift,X,y);
  FDown := False;
  if FShowClick then
    PaintFrame;
end;

procedure TImageBox.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  inherited MouseDown(Button,Shift,X,Y);
  FDown := True;
  if FShowClick then
    PaintFrame;
end;

procedure TImageBox.CMMouseEnter(var Message: TMessage);
begin
  inherited;
  if Assigned(FOnEnter) then FOnEnter(self);
  if TmpColor <> FHiColor then
  begin
    TmpColor := FHiColor;
    Repaint;
  end;
end;

procedure TImageBox.CMMouseLeave(var Message: TMessage);
begin
  inherited;
  FDown := False;
  if Assigned(FOnExit) then FOnExit(self);
  if TmpColor <> FBkCOlor then
  begin
    TmpColor := FBkColor;
    Repaint;
  end;
end;

procedure Register;
begin
  RegisterComponents('Add-Ons', [TImageWindow,TImageBox]);
end;

end.
